require "spec_helper"

module WikiTipsPost

  describe WikiPostHelper do
    let!(:tmp_dir) { Dir.mktmpdir }
    let!(:kernel) { double(:kernel_wrapper) }
    let(:subject) do
      described_class.new(
        :title => 'test title',
        :tips => 'test tips text',
        :tags => ['test_tag1', 'test_tag2']
      )
    end

    before do
      allow(KernelWrapper).to receive(:new).and_return(kernel)
      allow(Dir).to receive(:mktmpdir).and_return(tmp_dir)
    end

    describe "adding a tip" do
      before do
        FileUtils.cp_r fixture_path, tmp_dir
      end

      after do
        FileUtils.rm_r tmp_dir
      end

      context 'index without tag' do
        let(:fixture_path) { path_for_fixture('wiki/index_without_tag/.') }

        it 'prepares the git repo in a tmp directory' do
          expect(kernel).to receive(:system).with("git clone git@github.com:pivotal-cloudops/cloudops-tools.wiki.git #{tmp_dir}")
          subject.prepare_repo
        end

        it 'adds new pages into the repo with subject and name' do
          subject.add_new_page

          tip_contents = File.read File.join(tmp_dir, 'test-title.md')
          expect(tip_contents).to eq('test tips text')
        end

        describe 'creates tag pages' do
          it 'when there is a space in between' do
            need_index =  subject.link_to_tag('test tag1')
            tag_page_contents = File.read File.join(tmp_dir, '#test-tag1.md')
            expect(tag_page_contents).to eq("\n* [[ test title ]]")
            expect(need_index).to be(true)
          end

          it 'when there is a space in front' do
            need_index =  subject.link_to_tag(' test tag2')
            tag_page_contents = File.read File.join(tmp_dir, '#test-tag2.md')
            expect(tag_page_contents).to eq("\n* [[ test title ]]")
            expect(need_index).to be(true)
          end

          it 'when there is a space in back' do
            need_index =  subject.link_to_tag('test tag3 ')
            tag_page_contents = File.read File.join(tmp_dir, '#test-tag3.md')
            expect(tag_page_contents).to eq("\n* [[ test title ]]")
            expect(need_index).to be(true)
          end
        end

        it 'adds the created and sorted tag page to the index page' do
          expected_content = (<<EOF).chomp
Category with Tags:
* [[ #test_taga ]]
* [[ #test_tagb ]]
* [[ #test_tagc ]]
EOF


          subject.link_to_index('test tagc')
          subject.link_to_index('test tagb')
          subject.link_to_index('test taga')

          index_page_contents = File.read File.join(tmp_dir, INDEX_PAGE)
          expect(index_page_contents).to eq(expected_content)
        end
      end

      context 'index with tag' do
        let(:fixture_path) { path_for_fixture('wiki/index_with_tag/.') }

        it 'add itself into a existing tag page' do
          tag_file_path = File.join(tmp_dir, '#test_tag1.md')
          expect(File.exist?(tag_file_path))

          need_index =  subject.link_to_tag('test_tag1')
          tag_page_contents = File.read File.join(tmp_dir, '#test_tag1.md')
          expect(tag_page_contents).to eq("* [[ some_tip ]]\n* [[ test title ]]")
          expect(need_index).to be(false)
        end

        it 'adds the created tag page to the index page' do
          expected_content = "Category with Tags:\n* [[ #test_tag1 ]]\n"

          # Nothing changes before and after
          index_page_contents = File.read File.join(tmp_dir, INDEX_PAGE)
          expect(index_page_contents).to eq(expected_content)

          subject.link_to_index('test_tag1')

          index_page_contents = File.read File.join(tmp_dir, INDEX_PAGE)
          expect(index_page_contents).to eq(expected_content)
        end
      end
    end


    describe "pushing to the wiki repo" do
      it "calls git push from the tmp dir" do
        expect(kernel).to receive(:system).with("pushd #{tmp_dir} && git add . && git commit -m'Added a tip' && git push origin head && popd")
        subject.push_to_repo
      end
    end

    describe "deleting the temporary repo" do
      it "calls removes the tmp dir" do
        expect(kernel).to receive(:system).with(/^(sudo )?rm -r #{tmp_dir}/)
        subject.delete_local_repo_clone
      end
    end
  end
end
